package eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;

import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.BadPositionCategoryException;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentPartitioner;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.jface.text.TypedPosition;
import org.eclipse.jface.text.rules.FastPartitioner;
import org.eclipse.jface.text.rules.IPartitionTokenScanner;


public class NamePartitioner__ implements IDocumentPartitioner {

	private static final String NAME_CONTENT_TYPES_CATEGORY = "__name_content_types_category";
	/**
	 * The position category this partitioner uses to store the document's partitioning information.
	 */
	private final String fPositionCategory;
	private NameDocument fDocument;
	private String[] contentTypes = {"genus","specificEpithet"};
	
	public NamePartitioner__() {
		fPositionCategory = NAME_CONTENT_TYPES_CATEGORY + hashCode();
	}

	public ITypedRegion[] computePartitioning(int offset, int length) {
		// TODO Auto-generated method stub
		return null;
	}

	public void connect(IDocument document) {
		Assert.isTrue(document instanceof NameDocument, 
				"NamePartitioner__ requires a NameDocument");
		this.fDocument = (NameDocument) document;
		this.fDocument.addPositionCategory(fPositionCategory);
		
		TaxonName tn = fDocument.getTaxonName();
		
		if (tn == null) {
			// find matching pattern to init TaxonName
			
			document.get(); 
		} else {
			// construct partitions from TaxonName
						
			try {
				TypedPosition[] positions = new TypedPosition[contentTypes.length];
				int offset = 0;
				String text = "";
				
				for (int i = 0; i < contentTypes.length; i++) {
					String field = contentTypes[i];
					
					IObservableValue observeValue = BeansObservables.observeValue(tn, field);
					
					String value = (String) observeValue.getValue();
					int length = value.length();
										
					TypedPosition p = new TypedPosition(offset, length, field);
					positions[i] = p;
					
					offset += length + 1;
					text += value + " ";	
				}
				fDocument.set(text);
				for (int i = 0; i < positions.length; i++)
					fDocument.addPosition(fPositionCategory, positions[i]);
//			} catch (BadLocationException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			} catch (SecurityException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			} catch (NoSuchFieldException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			} catch (IllegalArgumentException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			} catch (IllegalAccessException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			} catch (BadPositionCategoryException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
			} catch (Exception e) {
				e.printStackTrace();
				System.out.println(e.getMessage());
			}
		}
	}

	public void disconnect() {
		// TODO Auto-generated method stub
		
	}

	public void documentAboutToBeChanged(DocumentEvent event) {
		System.out.println("You are here: " + getContentType(event.fOffset));
//		System.out.println("You are here: " + event.fOffset);
	}

	public boolean documentChanged(DocumentEvent event) {
		// TODO Auto-generated method stub
		return false;
	}

	public String getContentType(int offset) {
		ITypedRegion region = null;
		try {
			region = fDocument.getPartition(offset);
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return region.getType();
	}

	public String[] getLegalContentTypes() {
		return contentTypes;
	}

	public ITypedRegion getPartition(int offset) {
		ITypedRegion region = null;
		try {
			region = fDocument.getPartition(offset);
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return region;
	}
}
