package eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport;

import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

public class NameViewerConfig extends SourceViewerConfiguration {

	/** 
	 * Whenever a change is made to the document, a notification is sent to the 
	 *  IPresentationDamager for each content type in the affected area of the document. 
	 *  The IPresentationDamager in turn returns an IRegion indicating the area of the 
	 *  document which needs to be rebuilt as a result of the change. This information is 
	 *  then passed on to the IPresentationRepairer, which is responsible for reapplying 
	 *  the textual presentation for the affected region.
	 *  
	 *  This all probably sounds quite complicated. Fortunately you probably won't ever need 
	 *  to implement any of these classes yourself. JFace Text provides a 
	 *  DefaultDamagerRepairer which does the job both of the IPresentationDamager and the 
	 *  IPresentationRepairer. It also provides a PresentationReconciler class which performs 
	 *  the co-ordination role we described earlier.
	 *  
	 *  ...
	 *  
	 *  If you understand document partitioning then you are in a very good position to understand 
	 *  syntax highlighting in a JFace Text application. Essentially, syntax highlighting involves 
	 *  dividing a region into tokens, each of which has its own text display attributes. The 
	 *  standard mechanism for doing this also involves the use of the already familiar tokens, 
	 *  scanners and rules.
	 *   
	 **/  
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer)
	{
	    PresentationReconciler reconciler = new PresentationReconciler();
	    
	    DefaultDamagerRepairer dr = new NameDamagerRepairer(new NamePartitionScanner());
//	    reconciler.setDamager(dr, "genus");
//	    reconciler.setRepairer(dr, "genus");	    
	    
	    return reconciler;
	}
}
