// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.description;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.model.LanguageUtil;

/**
 * @author p.ciardelli
 * @created 02.04.2009
 * @version 1.0
 */
public class LanguageDialog extends InputDialog {
	private static final Logger logger = Logger.getLogger(LanguageDialog.class);
	
	private String message;
	
	private Language language;
	protected Language languageTemp;
	
	/**
	 * Language selected last time the dialog was used.
	 */
	private static Language lastLanguage;

	private CCombo combo;

	private List<Language> languages;


	/**
	 * @param shell
	 * @param dialogTitle
	 * @param dialogMessage
	 */
	public LanguageDialog(Shell shell, String dialogTitle, String dialogMessage) {
		super(shell, dialogTitle, 
				dialogMessage, null, null);
		
		message = dialogMessage;
		
		// Set selection menu to language chosen last time Dialog was opened 
		if (lastLanguage != null)  {
			language = lastLanguage;
		}
	}
	
	/**
	 * Lifted from Dialog.
	 * 
	 * @param parent
	 * @return
	 * @see Dialog
	 */
	protected Control createDialogComposite(Composite parent) {
		// create a composite with standard margins and spacing
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_MARGIN);
		layout.marginWidth = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_MARGIN);
		layout.verticalSpacing = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_SPACING);
		layout.horizontalSpacing = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_SPACING);
		composite.setLayout(layout);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		applyDialogFont(composite);
		return composite;
	}
	
    /*
     * (non-Javadoc) Method declared on Dialog.
     */
    protected Control createDialogArea(Composite parent) {
        // create composite
        Composite composite = (Composite) createDialogComposite(parent);
        // create message
        if (message != null) {
            Label label = new Label(composite, SWT.WRAP);
            label.setText(message);
            GridData data = new GridData(GridData.GRAB_HORIZONTAL
                    | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                    | GridData.VERTICAL_ALIGN_CENTER);
            data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
            label.setLayoutData(data);
            label.setFont(parent.getFont());
        }
        // create combo
        combo = new CCombo(composite, SWT.BORDER);
        combo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
        combo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int i = combo.getSelectionIndex();
				languageTemp = languages.get(i);
			}
		});
        
        populateLanguages();

        applyDialogFont(composite);
        return composite;
    }

	/**
	 * 
	 */
	private void populateLanguages() {
		languages = new ArrayList<Language>();
		int i = 0;		
		int index = 0;
		for (Language language : CdmSessionDataRepository.getDefault().getLanguages()) {
						
			combo.add(LanguageUtil.getDescription(language));
			languages.add(language);
			
			if (this.language != null && this.language.equals(language)) {
				index = i;  
			}
			i++;
		}
		combo.select(index);
	}

	/**
	 * @return
	 */
	private Language getLanguage() {
		return language;
	}
    
	/**
	 * @param dialogTitle 
	 * @param dialogMessage 
	 * @return
	 */
	public static Language getLanguage(String dialogTitle,
			String dialogMessage) {
		LanguageDialog dialog = new LanguageDialog(GlobalController.getShell(), 
				dialogTitle, 
				dialogMessage);
		if (dialog.open() == Window.CANCEL) {
			return null;
		}
		return dialog.getLanguage();
	}
	
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
    }
	
	protected void buttonPressed(int buttonId) {
		if (IDialogConstants.OK_ID == buttonId) {
			okPressed();
		} else if (IDialogConstants.CANCEL_ID == buttonId) {
			cancelPressed();
		}
	}
	
	protected void okPressed() {
		language = getLanguageSelection();
		lastLanguage = language;		
		super.okPressed();
	}
	
	protected void cancelPressed() {
		language = null;
		super.cancelPressed();
	}
	
	private Language getLanguageSelection() {
		int i = combo.getSelectionIndex();
		return languages.get(i);
	}
}
