/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations.name;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operations.AbstractEditorOperation;

/**
 * @author p.ciardelli
 * @created 14.01.2009
 * @version 1.0
 */
public class RemoveSynonymOperation extends AbstractEditorOperation {
	private static final Logger logger = Logger
			.getLogger(RemoveSynonymOperation.class);
	
	private Synonym synonym;
	private Set<SynonymRelationshipType> synonymTypes;

	public RemoveSynonymOperation(String label, IUndoContext undoContext,
			Taxon taxon, Synonym synonym) {
		super(label, undoContext, taxon);
		this.synonym = synonym;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Store synonymRelationshipType for later undo operations
		synonymTypes = synonym.getRelationType(taxon);
		
		// Remove synonym from taxon
		taxon.removeSynonym(synonym);
		
		// Redraw editor if exists
		return redrawOpenEditor();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {		
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Add synonym to taxon
		for (SynonymRelationshipType synonymType : synonymTypes){
			taxon.addSynonym(synonym, synonymType);
		}
		
		// Redraw editor if exists
		return redrawOpenEditor();
	}
}
