/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.reference;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Set;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.reference.Article;
import eu.etaxonomy.cdm.model.reference.Book;
import eu.etaxonomy.cdm.model.reference.BookSection;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.editor.reference.IReferenceSearch;

/**
 * Subclass of <code>ReferencePropertySource</code> which restricts reference type menu 
 * to "Article", "Generic", "Book", "Book Section".
 *  
 * @author p.ciardelli
 * @created 12.11.2008
 * @version 1.0
 */
public class NomenclaturalReferencePropertySource extends
		ReferencePropertySource {
	private static final Logger logger = Logger
			.getLogger(NomenclaturalReferencePropertySource.class);
	
	public NomenclaturalReferencePropertySource(ReferenceBase reference) {
		super(reference);
	}
	
	protected static HashMap<Class, String> nomReferenceTypeMap = null;
	
	protected void populateReferenceTypes() {
		
		// LinkedHashMap maintains insertion order
		nomReferenceTypeMap = new LinkedHashMap<Class, String>();
		
		nomReferenceTypeMap.put(ReferenceBase.class, "");
//		referenceTypeMap.put(BibtexReference.class, "BibTeX Reference");
		nomReferenceTypeMap.put(Article.class, "Article");
		nomReferenceTypeMap.put(Generic.class, "Generic");
		nomReferenceTypeMap.put(Book.class, "Book");
		nomReferenceTypeMap.put(BookSection.class, "Book Section");
	}
	
	protected String[] getReferenceTypeArray() {
		if (nomReferenceTypeMap == null) {
			populateReferenceTypes();
		}
		return nomReferenceTypeMap.values().toArray(new String[] {});
	}
	
	protected Set<Class> getReferenceClassSet() {
		return nomReferenceTypeMap.keySet();
	}
	
	protected int getSearchType() {
		return IReferenceSearch.NOMREF;
	}
}
