// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.reference.StrictReferenceBase;
import eu.etaxonomy.taxeditor.annotatedlineeditor.EntityListener;
import eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy;
import eu.etaxonomy.taxeditor.propertysheet.name.NonViralNamePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferencePropertySource;

/**
 * @author p.ciardelli
 * @created 07.07.2009
 * @version 1.0
 */
public class BulkEditorLineDisplay implements ILineDisplayStrategy {
	private static final Logger logger = Logger
			.getLogger(BulkEditorLineDisplay.class);
	
	private IEditorPart editor;

	/**
	 * @param bulkEditor
	 */
	public BulkEditorLineDisplay(IEditorPart editor) {
		this.editor = editor;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#getIcon(java.lang.Object)
	 */
	public Image getIcon(Object entity) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#getIndent(java.lang.Object)
	 */
	public int getIndent(Object entity) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#getPropertySource(java.lang.Object)
	 */
	public Object getPropertySource(Object entity) {
		if (entity instanceof ReferenceBase) {
			return new ReferencePropertySource((ReferenceBase) entity);			
		}
		if (entity instanceof NonViralName) {
			return new NonViralNamePropertySource((NonViralName) entity);
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#getText(java.lang.Object)
	 */
	public String getText(Object entity) {		
		if (entity instanceof ReferenceBase) {
			ReferenceBase reference = (ReferenceBase) HibernateProxyHelper.deproxy(entity);
			String text = reference.getTitleCache();
			if (text == null) {
				return "";
			}
			// Nobody seems to like displaying the date
//			TimePeriod datePublished = ((StrictReferenceBase) reference).getDatePublished();
//						
//			String date = datePublished == null ? null : datePublished.toString();
//			if (date != null) {
//				text += " [" + date + "]";
//			}
			return text;
		}
		if (entity instanceof TaxonNameBase) {
			String text = ((TaxonNameBase) entity).getTitleCache(); 
			return text == null ? "" : text;
		}
		return "";
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#setText(java.lang.String, java.lang.Object)
	 */
	public void setText(String text, Object entity) {
		if (entity instanceof ReferenceBase) {
			((ReferenceBase) entity).setTitleCache(text);			
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#getHoverText(java.lang.Object)
	 */
	public String getHoverText(Object entity) {
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#setStatusMessage(java.lang.String, java.lang.Object)
	 */
	public void setStatusMessage(final String text, Object entity) {
		
		// Calling from the synchronized method ListEditorReconcilingStrategy#doReconcile
		// without its own thread causes an invalid thread access exception
		// see http://wiki.eclipse.org/FAQ_Why_do_I_get_an_invalid_thread_access_exception%3F
		new Thread(new Runnable() {
			public void run() {
				Display.getDefault().asyncExec(new Runnable() {
					public void run() {
						IStatusLineManager statusLineManager = editor.getEditorSite().getActionBars().getStatusLineManager();
						statusLineManager.setMessage(text);
					}
				});
			}
		}).start();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy#getEmptyText(java.lang.Object)
	 */
	public String getEmptyCacheMessage(Object entity) {
		if (entity instanceof ReferenceBase) {
			return "No reference title cache";
		}
		if (entity instanceof TaxonNameBase) {
			return "No name title cache";
		}
		return "";
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy#isEntityCacheEmpty(java.lang.Object)
	 */
	public boolean isEntityCacheEmpty(Object entity) {
		if (entity instanceof ReferenceBase) {
			String text = ((ReferenceBase) entity).getTitleCache();
			return (text == null || text.equals(""));
		}
		if (entity instanceof TaxonNameBase) {
			String text = ((TaxonNameBase) entity).getTitleCache(); 
			return (text == null || text.equals(""));
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy#addDisplayListener(java.lang.Object)
	 */
	public void addDisplayListener(Object entity, EntityListener listener) {
		if (entity instanceof ReferenceBase) {
//			((ReferenceBase) entity).addPropertyChangeListener("titleCache", listener);
			((ReferenceBase) entity).addPropertyChangeListener(listener);
		}
		if (entity instanceof TaxonNameBase) {
			((TaxonNameBase) entity).addPropertyChangeListener("titleCache", listener);
		}
	}
}
