// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.reference.StrictReferenceBase;
import eu.etaxonomy.cdm.strategy.merge.MergeException;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IdentifiableEntityComparator;
import eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.NameEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.ReferenceEditorInput;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 07.07.2009
 * @version 1.0
 */
public class BulkEditorPersistenceService implements IEntityPersistenceService {
	private static final Logger logger = Logger
			.getLogger(BulkEditorPersistenceService.class);
	
	
	public ReferenceBase makeTestReference(String titleCache, int year) {
		StrictReferenceBase ref = (StrictReferenceBase) creator.createEntity(titleCache);
		if (year > 0)
			ref.setDatePublished(TimePeriod.NewInstance(year));
		return ref;
	}

	ReferenceCreator creator = new ReferenceCreator();
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityPersistenceService#getEntityList(java.lang.Object)
	 */
	public List<?> getEntityList(Object element) {
		List entityList = new ArrayList();
		
		if (element instanceof BulkEditorInput) {
			IBulkEditorQuery query = ((BulkEditorInput) element).getQuery();
			
			Comparator queryComparator = null;
			if (query != null) {
				String queryString = query.getSearchString();
				IIdentifiableEntityServiceConfigurator configurator = query.getSearchConfigurator();
				queryComparator = query.getComparator();
				if (element instanceof ReferenceEditorInput) {
					entityList = CdmStore.findReferences(configurator);
				}
				if (element instanceof NameEditorInput) {
					entityList = CdmStore.findNames(configurator);
				}
				// TODO enable when implementing AgentEditor
//				if (element instanceof AgentEditorInput){
//					entityList = CdmStore.findAgents(configurator);
//				}
				if (queryComparator != null) {
					Collections.sort(entityList, queryComparator);				
				} else {
					Collections.sort(entityList, new IdentifiableEntityComparator());
				}
				((BulkEditorInput) element).setQuery(null);
			}
//			entityList.add(makeTestReference("Acta universitatis carolinae. Biologica Prague",1932));
//			entityList.add(makeTestReference("Acta universitatis asiae mediae. Botanica",1753));
//			entityList.add(makeTestReference("Acta phytogeographica suecica Uppsala",1900));
//			entityList.add(makeTestReference("Acta botanica neerlandica Amsterdam",1864));
//			entityList.add(makeTestReference("Acta geobotanica hungarica Debrecen, Hungary",0));
		}
		return entityList;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityPersistenceService#create(java.lang.Object, java.lang.Object)
	 */
	public boolean create(Object entity, Object element) {
		logger.info("Create "  + entity);
		return save(entity, element);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityPersistenceService#delete(java.lang.Object, java.lang.Object)
	 */
	public boolean delete(Object entity, Object element) {
		logger.info("Delete "  + entity);
		if (entity instanceof ReferenceBase) {
			CdmStore.getReferenceService().delete((ReferenceBase) entity);
		}
		if (entity instanceof TaxonNameBase) {
			CdmStore.getNameService().delete((TaxonNameBase) entity);			
		}
		return true;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityPersistenceService#merge(java.lang.Object, java.lang.Object, java.lang.Object)
	 */
	public boolean merge(Object entity, Object mergeTarget, Object element) {
		logger.info("Merge "  + entity + " into " + mergeTarget);
		if (entity instanceof ReferenceBase) {
			try {
				CdmStore.getCommonService().merge((ReferenceBase) mergeTarget, (ReferenceBase) entity, null);
			} catch (MergeException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if (entity instanceof TaxonNameBase) {
			logger.warn("Merging not yet implemented for taxon name base");			
		}
		return true;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityPersistenceService#save(java.lang.Object, java.lang.Object)
	 */
	public boolean save(Object entity, Object element) {
		logger.info("Save "  + entity);
		if (entity instanceof ReferenceBase) {
			CdmStore.getReferenceService().saveOrUpdate((ReferenceBase) entity);			
		}
		if (entity instanceof TaxonNameBase) {
			CdmStore.getNameService().saveOrUpdate((TaxonNameBase) entity);				
		}
		return true;
	}
}
