// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import java.util.Comparator;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IEditorInput;

import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.reference.StrictReferenceBase;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IdentifiableEntityComparator;
import eu.etaxonomy.taxeditor.bulkeditor.input.ReferenceEditorInput;

/**
 * @author p.ciardelli
 * @created 19.08.2009
 * @version 1.0
 */
public class BulkEditorSortMenuProvider implements IBulkEditorSortMenuProvider {
	private static final Logger logger = Logger
			.getLogger(BulkEditorSortMenuProvider.class);

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorSortMenuProvider#fillMenu(org.eclipse.ui.IEditorInput, java.awt.Menu)
	 */
	public Menu fillMenu(IEditorInput input, Menu menu) {
		addMenuItem(menu, "Title cache", new IdentifiableEntityComparator(), true);
		
		if (input instanceof ReferenceEditorInput) {
			addMenuItem(menu, "Year", new ReferenceYearComparator(), false);
		}

		if (input instanceof ReferenceEditorInput) {
			addMenuItem(menu, "Reference Type", new ReferenceTypeComparator(), false);
		}
		
		return menu;
	}
	
	/**
	 * @param menu 
	 * @param string
	 * @param identifiableEntityComparator
	 * @return 
	 */
	private MenuItem addMenuItem(Menu menu, String text, Comparator comparator, boolean selected) {
		MenuItem menuItem = new MenuItem(menu, SWT.CHECK);
		menuItem.setText(text);
		menuItem.setData(comparator);
		menuItem.setSelection(selected);
		return menuItem;
	}
	
	class ReferenceYearComparator implements Comparator {
		
		/**
		 * @param o1
		 * @return
		 */
		private String getYearString(Object o) {
			TimePeriod datePublished = o == null || !(o instanceof ReferenceBase) ? 
					null : ((StrictReferenceBase) o).getDatePublished(); 
			return datePublished == null? null : datePublished.toString();
		}
		
		/* (non-Javadoc)
		 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
		 */
		public int compare(Object o1, Object o2) {
			String yearString1 = getYearString(o1);
			String yearString2 = getYearString(o2);
			if (yearString1 == null) {
				if (yearString2 == null) {
					return 0;
				} else {
					return -1;
				}
			}
			if (yearString2 == null) {
				return 1;
			}
			int returnVal = yearString1.compareToIgnoreCase(yearString2);
			if (returnVal == 0) {
				return new IdentifiableEntityComparator().compare(o1, 02);
			} else {
				return returnVal;
			}
		}		
	}
	
	class ReferenceTypeComparator implements Comparator {
		
		/* (non-Javadoc)
		 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
		 */
		public int compare(Object o1, Object o2) {
			String typeString1 = o1.getClass().toString();
			String typeString2 = o2.getClass().toString();
			if (typeString1 == null) {
				if (typeString2 == null) {
					return 0;
				} else {
					return -1;
				}
			}
			if (typeString2 == null) {
				return 1;
			}
			int returnVal = typeString1.compareToIgnoreCase(typeString2);
			if (returnVal == 0) {
				return new IdentifiableEntityComparator().compare(o1, 02);
			} else {
				return returnVal;
			}
		}		
	}
}