// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.ui.IElementFactory;
import org.eclipse.ui.IMemento;

import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * @author p.ciardelli
 * @created 23.04.2009
 * @version 1.0
 */
public class TaxonEditorInputFactory implements IElementFactory {
	private static final Logger logger = Logger
			.getLogger(TaxonEditorInputFactory.class);

    /**
     * Factory id. The workbench plug-in registers a factory by this name
     * with the "org.eclipse.ui.elementFactories" extension point.
     */
    private static final String ID_FACTORY = "eu.etaxonomy.taxeditor.editor.TaxonEditorInputFactory"; //$NON-NLS-1$
	
    /**
     * Tag for the IFile.fullPath of the file resource.
     */
    private static final String TAXON_NODE_UUID = "uuid"; //$NON-NLS-1$
    
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IElementFactory#createElement(org.eclipse.ui.IMemento)
	 */
	public IAdaptable createElement(IMemento memento) {
        // Get the uuid
        String taxonNodeUuid = memento.getString(TAXON_NODE_UUID);
        if (taxonNodeUuid == null || CdmStore.getTaxonService().getTaxonNodeByUuid(UUID.fromString(taxonNodeUuid)) == null) {
        	logger.warn("Couldn't find taxon node with UUID " + taxonNodeUuid);
//        	return MementoOutOfSyncInput.NewInstance(memento);
        	return null;
        }
        logger.info("Creating taxon node from memento w UUID " + taxonNodeUuid);
        return TaxonEditorInput.NewInstance(UUID.fromString(taxonNodeUuid));
	}
	
    /**
     * Returns the element factory id for this class.
     * 
     * @return the element factory id
     */
    public static String getFactoryId() {
        return ID_FACTORY;
    }
    
    /**
     * Saves the state of the given file editor input into the given memento.
     *
     * @param memento the storage area for element state
     * @param input the file editor input
     */
    public static void saveState(IMemento memento, TaxonEditorInput input) {
    	String uuid = input.getTaxonNode().getUuid().toString();
        memento.putString(TAXON_NODE_UUID, uuid);
    }
}