/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.taxeditor.model.DescriptionHelper;

/**
 * @author p.ciardelli
 *
 */
public class DescriptionLabelProvider extends LabelProvider implements IStyledLabelProvider  {
//public class DescriptionLabelProvider extends StyledCellLabelProvider {	
	private static final Logger logger = Logger
			.getLogger(DescriptionLabelProvider.class);

	public String getText(Object element) {
		// TODO move to DescriptiveInformationProvider?
		if (element instanceof TaxonDescription) {
			return getTaxonDescriptionText((TaxonDescription) element);
		}
		if (element instanceof TaxonDescriptionFeature) {
			return getFeatureText((TaxonDescriptionFeature) element);
		}
		if (element instanceof CommonTaxonName) {
			return getCommonNameText((CommonTaxonName) element);			
		}
		if (element instanceof Distribution) {
			return getDistributionText((Distribution) element);			
		}		
		if (element instanceof DescriptionElementBase) {
			return getElementText((DescriptionElementBase) element);
		}
		if (element instanceof ImageFile) {
			return getImageText((ImageFile) element);
		}
		return element.toString();
	}	
	
	/**
	 * @param element
	 * @return
	 */
	private String getDistributionText(Distribution element) {
		
		String text = "No text provided";
		
		Distribution distribution = (Distribution) element;
		
		NamedArea area = distribution.getArea();
		PresenceAbsenceTermBase<?> status = distribution.getStatus();
		
		if (status != null) {
			text = area.getLabel() + ", " + status.getLabel();
		}
		return text;
	}

	/**
	 * @param element
	 * @return
	 */
	private String getImageText(ImageFile imageFile) {
		String uri = imageFile.getUri();
		if (uri == null || uri.length() == 0) {
			uri = "No URL provided.";
		}
		return uri;
	}

	/**
	 * @param element
	 * @return
	 */
	private String getElementText(DescriptionElementBase element) {
		String text = DescriptionHelper.getCache(element);
		boolean isImageGallery = element.getInDescription().isImageGallery();
		if (text == null || text.length() == 0) {
			text = isImageGallery ? "No label provided" : "No text provided";
		}
		if (isImageGallery) {
			text = "Ordered image list: " + text;
		}
		return text;
	}

	/**
	 * @param element
	 * @return
	 */
	private String getCommonNameText(CommonTaxonName commonName) {
		String name = commonName.getName();
		if (name == null || name.length() == 0) {
			name = "No name provided";
		}
		Language language = commonName.getLanguage();
		if (language == null) {
			name += " (No language provided)";
		} else {
			name += " (" + language.getDescription() + ")";
		}
		return name;
	}

	/**
	 * @param element
	 * @return
	 */
	private String getFeatureText(TaxonDescriptionFeature descriptionFeature) {
		return descriptionFeature.getFeature().getLabel();
	}

	/**
	 * @param element
	 * @return
	 */
	private String getTaxonDescriptionText(TaxonDescription description) {
		String text = description.getTitleCache();
		if (text == null || text.length() == 0) {
			text = "No label provided";
		}
		return (description.isImageGallery() ? "Image Gallery: " : "Description: ") + text;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider#getStyledText(java.lang.Object)
	 */
	public StyledString getStyledText(Object element) {
		return new StyledString(getText(element), StyledString.QUALIFIER_STYLER);
	}
}
