// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.descriptiontree.handler;
import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.model.ImagesHelper;
import eu.etaxonomy.taxeditor.operations.AddImageToDescriptionElementOperation;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.operations.MoveImageInListOperation;

/**
 * @author p.ciardelli
 * @created 11.08.2009
 * @version 1.0
 */
public class MoveImageDownInListHandler extends AbstractHandler {
	private static final Logger logger = Logger
			.getLogger(CreateDescriptionElementHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		if (editor instanceof FormEditor) {
			editor = ((FormEditor) editor).getActiveEditor();
		}
		IEditorInput input = editor.getEditorInput();
		if (input instanceof TaxonEditorInput) {
			Taxon taxon = ((TaxonEditorInput) input).getTaxon();
			
			ISelection selection = HandlerUtil.getCurrentSelection(event);
			if (selection instanceof TreeSelection) {
				TreePath[] paths = ((TreeSelection) selection).getPaths();
				int count = paths[0].getSegmentCount();
				DescriptionElementBase element = null;
				for (int i = 0; i < count; i++ ) {
					if (paths[0].getSegment(i) instanceof DescriptionElementBase) {
						element = (DescriptionElementBase) paths[0].getSegment(i);
						break;
					}
				}				
				ImageFile image = (ImageFile) paths[0].getLastSegment();
				IUndoableOperation operation = null;
				try {
					// TODO use undo context specific to editor
					operation = new MoveImageInListOperation(event.getCommand().getName(), 
							EditorUtil.getUndoContext(), taxon, element, image, ImagesHelper.DOWN, (IPostOperationEnabled) editor);
					EditorUtil.executeOperation(operation);
				} catch (NotDefinedException e) {
					logger.warn("Command name not set");
				}
			}
		} 
		return null;
	}

}
