// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.dialogs.FilteredNameSelectionDialog;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author p.ciardelli
 * @created 07.04.2009
 * @version 1.0
 */
public class NameSelectComposite extends Composite {
	
	private TaxonNameBase<?, ?> savedName;
	private Text txtName;
	private Button btnClearName;
	
	PropertyChangeSupport propertyChangeSupport;
	public static final String NAME = "name";
	
	/**
	 * @param parent
	 * @param style
	 */
	public NameSelectComposite(Composite parent) {
		super(parent, SWT.NULL);
		createContent(parent);
	}
	
	private void createContent(Composite container) {
		
		propertyChangeSupport = new PropertyChangeSupport(this);
		
		setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		GridLayout gridLayout = new GridLayout();
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		Label lblName = new Label(this, SWT.NONE);
		lblName.setLayoutData(new GridData());
		lblName.setText("Choose a name:");
		
		// Create 3-columned composite for name input
		Composite nameComposite = new Composite(this, SWT.NONE);
		GridLayout gridLayout2 = new GridLayout();
		gridLayout2.numColumns = 3;	
		gridLayout2.marginHeight = 0;
		gridLayout2.marginWidth = 0;
		nameComposite.setLayout(gridLayout2);
		nameComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		// Create name input
		txtName = new Text(nameComposite, SWT.BORDER);
		txtName.setEnabled(false);
		txtName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		txtName.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				notifyListeners();
			}
		});
		
		// Create name search button
		Button btnSearchName = new Button(nameComposite, SWT.NONE);
		btnSearchName.setEnabled(true);
		btnSearchName.setLayoutData(new GridData());
		btnSearchName.setText("Search ...");
		btnSearchName.addSelectionListener(new SelectionAdapter() {
			
			// Popup name search
			public void widgetSelected(SelectionEvent e) {
				popupSearch();
			}
		});

		// Create clear name button
		btnClearName = new Button(nameComposite, SWT.NONE);
		btnClearName.setEnabled(false);
		btnClearName.setText("Clear");
		btnClearName.addSelectionListener(new SelectionAdapter() {
			
			// Clear selected name
			public void widgetSelected(SelectionEvent e) {
				clearName();
			}
		});
	}
	
	/**
	 * @param nameExists
	 */
	protected void notifyListeners() {
		propertyChangeSupport.firePropertyChange(NAME, null, null);
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(NAME, listener);
	}
	
	public void setName(TaxonNameBase<?, ?> name) {
		savedName = name;
		txtName.setText(name.getTitleCache());
		txtName.setEditable(false);
					
		btnClearName.setEnabled(true);
	}
	
	/**
	 * 
	 */
	protected void popupSearch() {		
		TaxonNameBase name = FilteredNameSelectionDialog.selectName(getShell(), savedName);
		if(name != null){
			setSavedName(name);
		}
	}

	/**
	 * @param value
	 */
	private void setSavedName(TaxonNameBase<?, ?> name) {
		
		savedName = name;
		
		txtName.setText(name.getTitleCache());
		txtName.setEditable(false);
		
		btnClearName.setEnabled(true);
	}
	
	protected void clearName() {
		savedName = null;
		
		txtName.setText("");
		txtName.setEditable(true);
		
		btnClearName.setEnabled(false);
	}

	/**
	 * Returns name object, if any. Otherwise, creates a new
	 * name object using text the user has input. Zero-length name
	 * returns NULL.
	 *  
	 * @return
	 */
	public TaxonNameBase<?, ?> getName() {
		return savedName;
	}
}