// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.reference;

import java.util.UUID;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.dialogs.FilteredReferenceSelectionDialog;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferencePropertySource;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 07.04.2009
 * @version 1.0
 */
public class ReferenceSelectComposite extends Composite {
	private ReferenceBase<?> savedReference;
	private Text txtReference;
	private Button btnClearReference;
	private Text txtMicroRef;
	
	/**
	 * @param parent
	 * @param style
	 */
	public ReferenceSelectComposite(Composite parent) {
		super(parent, SWT.NULL);
		createContent(parent);
	}
	
	private void createContent(Composite xcontainer) {
		setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		GridLayout gridLayout = new GridLayout();
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		Label lblReference = new Label(this, SWT.NONE);
		lblReference.setLayoutData(new GridData());
		lblReference.setText("Choose a reference:");
		
		// Create 3-columned composite for reference input
		Composite refComposite = new Composite(this, SWT.NONE);
		GridLayout gridLayout2 = new GridLayout();
		gridLayout2.numColumns = 3;	
		gridLayout2.marginHeight = 0;
		gridLayout2.marginWidth = 0;
		refComposite.setLayout(gridLayout2);
		refComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		// Create reference input
		txtReference = new Text(refComposite, SWT.BORDER);
		txtReference.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		txtReference.setEnabled(false);
		
		// Create reference search button
		Button btnSearchReference = new Button(refComposite, SWT.NONE);
		btnSearchReference.setEnabled(true);
		btnSearchReference.setLayoutData(new GridData());
		btnSearchReference.setText("Search ...");
		btnSearchReference.addSelectionListener(new SelectionAdapter() {
			
			// Popup reference search
			public void widgetSelected(SelectionEvent e) {
				popupSearch();
			}
		});

		// Create clear reference button
		btnClearReference = new Button(refComposite, SWT.NONE);
		btnClearReference.setEnabled(false);
		btnClearReference.setText("Clear");
		btnClearReference.addSelectionListener(new SelectionAdapter() {
			
			// Clear selected reference
			public void widgetSelected(SelectionEvent e) {
				clearReference();
			}
		});
				
		Label lblMicroref = new Label(this, SWT.NONE);
		lblMicroref.setText("Enter reference detail (e.g., page no.):");
		
		// Create microref input
		txtMicroRef = new Text(this, SWT.BORDER);
		txtMicroRef.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false));
	}
	
	public void setReference(ReferenceBase<?> reference) {
		savedReference = reference;
		txtReference.setText(reference.getTitleCache());
		txtReference.setEditable(false);
					
		btnClearReference.setEnabled(true);
	}
	
	public void setMicroReference(String microref) {
		txtMicroRef.setText(CdmUtils.Nz(microref));
	}
	
	/**
	 * 
	 */
	protected void popupSearch() {
        ReferenceBase reference = FilteredReferenceSelectionDialog.selectReference(getShell(), savedReference);
		if (reference != null) {
			setSavedReference(reference);
		}
	}

	/**
	 * @param value
	 */
	private void setSavedReference(ReferenceBase<?> reference) {
		
		savedReference = reference;
		
		txtReference.setText(reference.getTitleCache());
		txtReference.setEditable(false);
		
		btnClearReference.setEnabled(true);
	}
	
	protected void clearReference() {
		savedReference = null;
		
		txtReference.setText("");
		txtReference.setEditable(true);
		
		btnClearReference.setEnabled(false);		
	}

	/**
	 * Returns reference object, if any. Otherwise, creates a new Generic
	 * reference object using text the user has input. 
	 * If no text, returns null.
	 *  
	 * @return
	 */
	public ReferenceBase<?> getReference() {
		return savedReference;
	}
	
	public String getMicroReference() {
		return txtMicroRef.getText();
	}
}