/**
 * 
 */
package eu.etaxonomy.taxeditor.propertysheet;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;

import eu.etaxonomy.cdm.model.common.Extension;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author p.ciardelli
 *
 */
public class ExtensionsDialog extends TitleAreaDialog {

	private IdentifiableEntity entity;
	private Button addButton;
	private TableViewer tableViewer;
	private Button removeButton;
	private Set<Extension> extensions = new HashSet<Extension>();
	private Button okButton;

	/**
	 * @param parentShell
	 * @param entity 
	 */
	public ExtensionsDialog(Shell parentShell, IdentifiableEntity entity) {
		super(parentShell);

		this.entity = entity;
		this.extensions.addAll(entity.getExtensions());		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#open()
	 */
	@Override
	public int open() {
		return super.open();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
	    Control contents = super.createContents(parent);
				
		setTitle("Edit extensions");
		setMessage("Add or remove extensions");

		this.getShell().setText("Extensions");
		
		return contents;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		
        // create composite
        Composite composite = new Composite(container, SWT.NULL);
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		composite.setLayout(gridLayout);
	
		tableViewer = new TableViewer(composite, SWT.NULL);
		GridData tableLayoutData = new GridData(SWT.FILL, SWT.FILL, true, false);
		tableLayoutData.heightHint = 300;
		tableViewer.getTable().setLayoutData(tableLayoutData);
		createColumns(tableViewer);
		
		tableViewer.setContentProvider(new ExtensionsContentProvider());
		tableViewer.setLabelProvider(new ExtensionsLabelProvider());
		
		tableViewer.addSelectionChangedListener(new ISelectionChangedListener () {
			public void selectionChanged(SelectionChangedEvent event) {
				setRemovedEnabled(true);
			}
		});
				
		Composite buttonContainer = new Composite(composite, SWT.NULL);
		GridData buttonContainerLayoutData = new GridData();
		buttonContainerLayoutData.verticalAlignment = SWT.TOP;
		buttonContainer.setLayoutData(buttonContainerLayoutData);
		buttonContainer.setLayout(new GridLayout());
		
		addButton = new Button(buttonContainer, SWT.PUSH);
		addButton.setText("Add ...");
		addButton.addMouseListener(new MouseAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.MouseAdapter#mouseDown(org.eclipse.swt.events.MouseEvent)
			 */
			@Override
			public void mouseUp(MouseEvent e) {
				createExtensions();
			}			
		});

		removeButton = new Button(buttonContainer, SWT.PUSH);
		removeButton.setText("Remove");
		removeButton.addMouseListener(new MouseAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.MouseAdapter#mouseUp(org.eclipse.swt.events.MouseEvent)
			 */
			@Override
			public void mouseUp(MouseEvent e) {
				removeSelectedExtension();
			}
		});
		setRemovedEnabled(false);
			
		populateExtensions();
		
		return container;
	}
	
	private void createColumns(TableViewer viewer) {
		Table table = viewer.getTable();
		String[] titles = {"Type", "Value"};
		int[] bounds = { 100, 200 };

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
	}
	
	/**
	 * 
	 */
	protected void createExtensions() {
		
        CreateExtensionDialog dialog = new CreateExtensionDialog(getShell());
		if (dialog.open() == Window.OK) {
			Extension extension = dialog.getNewExtension();
			if (extension != null) {
				extensions.add(extension);
				tableViewer.refresh();
			}
		}
	}

	/**
	 * 
	 */
	protected void removeSelectedExtension() {
		Extension extension = getSelectedExtension();
		if (extension != null) {
			extensions.remove(extension);
			tableViewer.refresh();
		}
	}

	/**
	 * @return 
	 * 
	 */
	private Extension getSelectedExtension() {
		return (Extension) ((IStructuredSelection) tableViewer.getSelection()).getFirstElement();
	}

	/**
	 * 
	 */
	protected void setRemovedEnabled(boolean enabled) {
		removeButton.setEnabled(enabled);		
	}

	/**
	 * 
	 */
	protected void toggleSelection() {
		// TODO Auto-generated method stub
		
	}

	/**
	 * 
	 */
	private void populateExtensions() {
		tableViewer.setInput(extensions);
	}


	private class ExtensionsContentProvider implements IStructuredContentProvider{

		public Object[] getElements(Object inputElement) {			
			return ((Set<?>) inputElement).toArray();
		}

		public void dispose() {}

		public void inputChanged(Viewer viewer, Object oldInput,
				Object newInput) {}
		
	};
	
	class ExtensionsLabelProvider extends LabelProvider implements ITableLabelProvider{

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			if (columnIndex == 0) {
				return ((Extension) element).getType().getLabel();
			}
			if (columnIndex == 1) {
				return ((Extension) element).getValue();
			}
			return null;
		}
	}
	
    protected void createButtonsForButtonBar(Composite parent) {
        okButton = createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    @Override
    protected void okPressed() {
		Set<Extension> savedExtensions = entity.getExtensions();
		for (Extension extension : extensions) {
			if (!savedExtensions.contains(extension)) {
				entity.addExtension(extension);
			}
		}
		Set<Extension> removeExtensions = new HashSet<Extension>();
		for (Extension extension : savedExtensions) {
			if (!extensions.contains(extension)) {
				removeExtensions.add(extension);
			}
		}
		for (Extension extension : removeExtensions) {
			entity.removeExtension(extension);
		}
    	super.okPressed();
    }
}
