/**
 * 
 */
package eu.etaxonomy.taxeditor.propertysheet;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author p.ciardelli
 *
 */
public class MarkersDialog extends TitleAreaDialog {

	private AnnotatableEntity entity;
	private Button addButton;
	private CheckboxTableViewer tableViewer;
	private Button removeButton;
	private Set<Marker> markers = new HashSet<Marker>();
	private Button okButton;

	/**
	 * @param parentShell
	 * @param entity 
	 */
	public MarkersDialog(Shell parentShell, AnnotatableEntity entity) {
		super(parentShell);

		this.entity = entity;			
		this.markers.addAll(entity.getMarkers());
		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#open()
	 */
	@Override
	public int open() {
		return super.open();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
	    Control contents = super.createContents(parent);
				
		setTitle("Edit markers");
		setMessage("Add or remove markers, and set their flags to true or false");

		this.getShell().setText("Markers");
		
		return contents;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		
        // create composite
        Composite composite = new Composite(container, SWT.NULL);
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		composite.setLayout(gridLayout);
	
		tableViewer = CheckboxTableViewer.newCheckList(composite, SWT.NULL);
		GridData tableLayoutData = new GridData(SWT.FILL, SWT.FILL, true, false);
		tableLayoutData.heightHint = 300;
		tableViewer.getTable().setLayoutData(tableLayoutData);
		
		tableViewer.setContentProvider(new MarkersContentProvider());
		tableViewer.setLabelProvider(new MarkersLabelProvider());
		
		tableViewer.addSelectionChangedListener(new ISelectionChangedListener () {
			public void selectionChanged(SelectionChangedEvent event) {
				setRemovedEnabled(true);
			}
		});
				
		Composite buttonContainer = new Composite(composite, SWT.NULL);
		GridData buttonContainerLayoutData = new GridData();
		buttonContainerLayoutData.verticalAlignment = SWT.TOP;
		buttonContainer.setLayoutData(buttonContainerLayoutData);
		buttonContainer.setLayout(new GridLayout());
		
		addButton = new Button(buttonContainer, SWT.PUSH);
		addButton.setText("Add ...");
		addButton.addMouseListener(new MouseAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.MouseAdapter#mouseDown(org.eclipse.swt.events.MouseEvent)
			 */
			@Override
			public void mouseUp(MouseEvent e) {
				createMarker();
			}			
		});

		removeButton = new Button(buttonContainer, SWT.PUSH);
		removeButton.setText("Remove");
		removeButton.addMouseListener(new MouseAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.MouseAdapter#mouseUp(org.eclipse.swt.events.MouseEvent)
			 */
			@Override
			public void mouseUp(MouseEvent e) {
				removeSelectedMarker();
			}
		});
		setRemovedEnabled(false);
			
		populateMarkers();
		
		return container;
	}
	
	
	/**
	 * 
	 */
	protected void createMarker() {
		Set<MarkerType> markerTypes = VocabularyStore.getMarkerTypes();
		
		// Remove marker types that are already spoken for
		for (Marker marker : markers) {
			markerTypes.remove(marker.getMarkerType());
		}
		
        SelectMarkerTypeDialog dialog = new SelectMarkerTypeDialog(getShell(), markerTypes);
		if (dialog.open() == Window.OK) {
			Set<MarkerType> selectedMarkerTypes = dialog.getMarkerTypes();
			if (selectedMarkerTypes != null) {
				for (MarkerType markerType : selectedMarkerTypes) {
					markers.add(Marker.NewInstance(markerType, false));
				}
				tableViewer.refresh();
			}
		}
	}

	/**
	 * 
	 */
	protected void removeSelectedMarker() {
		Marker marker = getSelectedMarker();
		if (marker != null) {
			markers.remove(marker);
			tableViewer.refresh();
		}
	}

	/**
	 * @return 
	 * 
	 */
	private Marker getSelectedMarker() {
		return (Marker) ((IStructuredSelection) tableViewer.getSelection()).getFirstElement();
	}

	/**
	 * 
	 */
	protected void setRemovedEnabled(boolean enabled) {
		removeButton.setEnabled(enabled);		
	}

	/**
	 * 
	 */
	protected void toggleSelection() {
		// TODO Auto-generated method stub
		
	}

	/**
	 * 
	 */
	private void populateMarkers() {
		List<Marker> markerFlags = new ArrayList<Marker>(); 
		for (Marker marker : markers) {
			if (marker.getFlag()) {
				markerFlags.add(marker);
			}
		}
		tableViewer.setInput(markers);
		tableViewer.setCheckedElements(markerFlags.toArray());
	}


	private class MarkersContentProvider implements IStructuredContentProvider{

		public Object[] getElements(Object inputElement) {			
			return ((Set<?>) inputElement).toArray();
		}

		public void dispose() {}

		public void inputChanged(Viewer viewer, Object oldInput,
				Object newInput) {}
		
	};
	
	class MarkersLabelProvider extends LabelProvider implements ITableLabelProvider{

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			return ((Marker)element).getMarkerType().getLabel();
		}
	}
	
    protected void createButtonsForButtonBar(Composite parent) {
        okButton = createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    @Override
    protected void okPressed() {
    	List<Marker> checkedMarkers = new ArrayList<Marker>();
		for (Object element : tableViewer.getCheckedElements()){
			checkedMarkers.add((Marker)element);
		}
		Set<Marker> savedMarkers = entity.getMarkers();
		for (Marker marker : markers) {

			// Set marker flags
			marker.setFlag(checkedMarkers.contains(marker));
			
			// Save any new markers
			if (!savedMarkers.contains(marker)) {
				entity.addMarker(marker);
			}
		}
		// Remove deleted markers from entity
		Set<Marker> removeMarkers = new HashSet<Marker>();
		for (Marker marker : savedMarkers) {
			if (!markers.contains(marker)) {
				removeMarkers.add(marker);
			}
		}
		for (Marker marker : removeMarkers) {
			entity.removeMarker(marker);
		}
    	super.okPressed();
    }
}
