// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.type;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.propertysheet.ICdmBasePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferencePropertySource;

/**
 * @author p.ciardelli
 * @created 28.04.2009
 * @version 1.0
 */
public class TypeDesignationPropertySource implements ICdmBasePropertySource {

	private TypeDesignationBase<?> typeDesignation;

    /**
     * Property unique keys
     */
	public static final String P_ID_CITATION = "citation";
	public static final String P_ID_MICROREF = "microref";

    /**
     * Property display keys
	 */
	public static final String P_CITATION = "Citation";
	public static final String P_MICROREF = "Microcitation";
	
	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	TypeDesignationPropertySource(TypeDesignationBase<?> typeDesignation) {
		this.typeDesignation = typeDesignation;
		
		addDescriptor(P_ID_CITATION);
		addDescriptor(P_ID_MICROREF);
	}
	
	public Object getEditableValue() {
		if (typeDesignation instanceof SpecimenTypeDesignation) {
			return ((SpecimenTypeDesignation) typeDesignation).getTypeSpecimen().getTitleCache();
		}
		
		if (typeDesignation instanceof NameTypeDesignation) {
			if (((NameTypeDesignation) typeDesignation).getTypeName() != null) {
				return ((NameTypeDesignation) typeDesignation).getTypeName().getTitleCache();
			}
		}		
		return null;
	}

	private void addDescriptor(String id) {
		// Name relations, listed in custom property descriptor
		if (id.equals(P_ID_CITATION)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_CITATION, P_CITATION));
		}
		if (id.equals(P_ID_MICROREF)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_MICROREF, P_MICROREF));
		}
	}
	
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	public Object getPropertyValue(Object id) {
		
		if (id.equals(P_ID_CITATION)) {
		
			ReferenceBase<?> reference = typeDesignation.getCitation();
			if (reference == null) {
				reference = Generic.NewInstance();
			}
			ReferencePropertySource referencePropertySource = new ReferencePropertySource(reference);
			referencePropertySource.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (evt.getNewValue() instanceof ReferenceBase) {	
						typeDesignation.setCitation((ReferenceBase<?>) evt.getNewValue());
					}
				}
			});
			return referencePropertySource;
		}
		
		if (id.equals(P_ID_MICROREF)) {
			return CdmUtils.Nz(typeDesignation.getCitationMicroReference());
		}
		
		return null;
	}

	public boolean isPropertySet(Object id) {
		return false;
	}

	public void resetPropertyValue(Object id) {}

	public void setPropertyValue(Object id, Object value) {
		if (id.equals(P_ID_MICROREF)) {
			typeDesignation.setCitationMicroReference((String) value);
		}
	}
		
	public boolean equals(Object object) {
		if (object == null) {
			return false;
		}
		if (toString() == null) {
			return false;
		}
		if (toString().equals(object.toString())) {
			return true;
		} else {
			return false;
		}
	}
}
