/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.type;

import java.util.Set;
import java.util.Vector;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.propertysheet.ICdmBasePropertySource;

/**
 * @author p.ciardelli
 * @created 11.02.2009
 * @version 1.0
 */
public class TypePropertySource implements ICdmBasePropertySource {
	
	/**
	 * The name whose types are being displayed.
	 */
	private TaxonNameBase<?, ?> name;
	
	/**
     * Property unique keys
     */
	public static final String P_ID_NAMETYPES = "nametypes";

    /**
     * Property display keys
     */
	public static final String P_NAMETYPES = "Name Types";	
	
	protected static final String[] TOP_LEVEL_PROPERTIES = new String[] {
		P_ID_NAMETYPES};
	
	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	public TypePropertySource(TaxonNameBase<?, ?> name) {
		this.name = name;
		
		addDescriptor(P_ID_NAMETYPES);
	}
	
	
	/**
	 * 
	 */
	private void addDescriptor(String id) {
		// Name relations, listed in custom property descriptor
		if (id.equals(P_ID_NAMETYPES)) {
			descriptors.addElement(
					new TypePropertyDescriptor(P_ID_NAMETYPES, P_NAMETYPES, name) {
						protected void saveTypes(Set set) {
							setPropertyValue(P_ID_NAMETYPES, set);
						}
					}
			);
		};
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {

	}
}
