// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Sep 10, 2009
 * @version 1.0
 */
public class FilteredAgentSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<AgentBase> {
	private static final Logger logger = Logger
			.getLogger(FilteredAgentSelectionDialog.class);
	
	/**
	 * Dialog can be initialized with an agent. Otherwise, leave
	 * agent == NULL.  
	 * 
	 * @param shell
	 * @param agent
	 * @return
	 */
	public static AgentBase selectAgent(Shell shell, AgentBase agent) {
		FilteredAgentSelectionDialog dialog = new FilteredAgentSelectionDialog(shell, 
				"Choose authors or institutions", false, agent);
		if (dialog.open() == Window.CANCEL) {
			return null;
		}
		
		UUID agentUuid = dialog.getSelectedUuidAndTitleCache().getUuid();
		if(agentUuid == null){
			return null;
		}	
		return dialog.getCdmObjectByUuid(agentUuid);
	}
	
	private Set<AgentBase> transientAgents = new HashSet<AgentBase>();
	
	/**
	* @param shell
	* @param title
	 * @param agent 
	*/
	protected FilteredAgentSelectionDialog(Shell shell, String title, boolean multi, AgentBase agent) {
		super(shell, title, multi, FilteredAgentSelectionDialog.class.getCanonicalName(), agent);
		
		ILabelProvider labelProvider = 	new FilteredCdmResourceLabelProvider();
		setListLabelProvider(labelProvider);
		setDetailsLabelProvider(new DetailsLabelProvider());
	}
	
	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	*/
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		Link link = new Link(parent, SWT.NONE);
		link.setText("Click link to create a new <A>author</A> or <A>author team</A>.");
		link.addListener (SWT.Selection, new Listener () {
			public void handleEvent(Event event) {
				InputDialog dialog = new InputDialog(getShell(), "Create " + event.text, "Enter new " + event.text, "", null);
				if (dialog.open() == Window.OK) {
					
					TeamOrPersonBase teamOrPerson = ("author").equals(event.text) ? Person.NewInstance() : Team.NewInstance();
					teamOrPerson.setTitleCache(dialog.getValue());
					teamOrPerson.setNomenclaturalTitle(dialog.getValue());
					
					addObjectToModel(teamOrPerson);
					setPattern(teamOrPerson);
				}
			}
		});
		return link;
	}
	
	/**
	 * @author p.ciardelli
	 * @created 18.09.2009
	 * @version 1.0
	 */
	public class DetailsLabelProvider extends LabelProvider {
		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
		 */
		public String getText(Object element) {
			AgentBase agent = getCdmObjectByUuid(((UuidAndTitleCache<AgentBase>) element).getUuid());
			if (agent instanceof INomenclaturalAuthor) {
				return "Nomenclatural title: '" + ((INomenclaturalAuthor) agent).getNomenclaturalTitle() + "'"; 
			} else {
				return "'" + agent.getTitleCache() + "' is not a nomenclatural author."; 
			}
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#getPersistentObect(java.util.UUID)
	 */
	@Override
	protected AgentBase getPersistentObject(UUID cdmUuid) {
		return CdmStore.getAgentService().getAgentByUuid(cdmUuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	@Override
	protected void initModel() {
		model = CdmStore.getAgentService().getUuidAndTitleCache();
	}
}