// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 04.06.2009
 * @version 1.0
 */
public class FilteredReferenceSelectionDialog extends AbstractFilteredCdmResourceSelectionDialog<ReferenceBase> {
	private static final Logger logger = Logger
			.getLogger(FilteredReferenceSelectionDialog.class);

	/**
	* @param reference 
	 * @param taxon
	* @return
	*/
	public static ReferenceBase selectReference(Shell shell, ReferenceBase reference) {
		FilteredReferenceSelectionDialog dialog = new FilteredReferenceSelectionDialog(shell, 
				"Choose a reference", false, reference);
		if (dialog.open() == Window.CANCEL) {
			return null;
		}
		
		UUID referenceUuid = dialog.getSelectedUuidAndTitleCache().getUuid();
		if(referenceUuid == null){
			return null;
		}	
		return dialog.getCdmObjectByUuid(referenceUuid); 
	}

	/**
	* @param shell
	* @param title
	 * @param reference 
	*/
	protected FilteredReferenceSelectionDialog(Shell shell, String title, boolean multi, ReferenceBase reference) {
		super(shell, title, multi, FilteredReferenceSelectionDialog.class.getCanonicalName(), reference);
		
		ILabelProvider labelProvider = new FilteredCdmResourceLabelProvider();
		setListLabelProvider(labelProvider);
		setDetailsLabelProvider(labelProvider);
	}
	
	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	*/
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		Link link = new Link(parent, SWT.NONE);
		link.setText("Click <A>here</A> to create a new reference.");
		link.addListener (SWT.Selection, new Listener () {
			public void handleEvent(Event event) {
				
				NewReferenceInputDialog dialog = new NewReferenceInputDialog(getShell());
								
				if (dialog.open() == Window.OK) {
					
//					TeamOrPersonBase teamOrPerson = ("author").equals(event.text) ? Person.NewInstance() : Team.NewInstance();
//					teamOrPerson.setTitleCache(dialog.getValue());
//					teamOrPerson.setNomenclaturalTitle(dialog.getValue());
//					
//					addObjectToModel(teamOrPerson);
//					setPattern(teamOrPerson);
				}
			}
		});
		return link;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	@Override
	protected ReferenceBase getPersistentObject(UUID cdmUuid) {
		return CdmStore.getReferenceService().getReferenceByUuid(cdmUuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	@Override
	protected void initModel() {
		model = CdmStore.getReferenceService().getUuidAndTitleCache();
	}
}