/**
 * 
 */
package eu.etaxonomy.taxeditor.dialogs;

import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 *
 */
public class FilteredTaxonNodeSelectionDialog extends AbstractFilteredCdmResourceSelectionDialog {	
	
	private static final Logger logger = Logger
			.getLogger(FilteredTaxonNodeSelectionDialog.class);

	
	/**
	 * @param taxon
	 * @return
	 */
	public static TaxonNode selectTaxonNode(Shell shell, String title, List<UUID> excludeTaxa) {
		FilteredTaxonNodeSelectionDialog dialog = new FilteredTaxonNodeSelectionDialog(shell, 
				title,
				excludeTaxa,
				false);
		if (dialog.open() == Window.CANCEL) {
			return null;
		}
		
		UUID taxonNodeUuid = dialog.getSelectedUuidAndTitleCache().getUuid();
		
		if(taxonNodeUuid != null){
			return CdmStore.getTaxonService().getTaxonNodeByUuid(taxonNodeUuid);
		}
		return null; 
	}

	private List<UuidAndTitleCache> allTaxonBases;

	private Combo taxonomicTreeSelectionCombo;

	private List<TaxonomicTree> taxonomicTrees;
	
	private TaxonomicTree selectedTaxonomicTree;
	
	private List<UUID> excludeTaxa;

	/**
	 * 
	 * 
	 * @param shell
	 * @param title
	 */
	protected FilteredTaxonNodeSelectionDialog(Shell shell, String title, List<UUID> excludeTaxa, boolean multi) {
		super(shell, title, multi, FilteredTaxonNodeSelectionDialog.class.getCanonicalName());
		
		this.excludeTaxa = excludeTaxa;
		taxonomicTrees = CdmStore.getTaxonService().listTaxonomicTrees(null, null, null, null);
		// default to first tree 
		// TODO this will be problematic and can only be seen as workaround 
		selectedTaxonomicTree = taxonomicTrees.iterator().next();
		
		ILabelProvider labelProvider = new FilteredCdmResourceLabelProvider();

		setListLabelProvider(labelProvider);
		setDetailsLabelProvider(labelProvider);
		
		allTaxonBases = CdmStore.getTaxonService().getTaxonNodeUuidAndTitleCacheOfAcceptedTaxaByTaxonomicTree(selectedTaxonomicTree);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#fillContentProvider(org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.AbstractContentProvider, org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.ItemsFilter, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	protected void fillContentProvider(AbstractContentProvider contentProvider,
			ItemsFilter itemsFilter, IProgressMonitor progressMonitor)
			throws CoreException {
		progressMonitor.beginTask("Looking for taxa", 1000); // Why no progres monitor?
		if(excludeTaxa == null){
			for(UuidAndTitleCache taxon : allTaxonBases){
				contentProvider.add(taxon, itemsFilter);
			}
		}else{
			for(UuidAndTitleCache taxon : allTaxonBases){
				if( ! excludeTaxa.contains(taxon.getUuid())){
					contentProvider.add(taxon, itemsFilter);
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		
		// TODO tree selection is disabled at the moment and the first tree will always be used. 
		// change this once we are working with multiple trees
//		return createTreeSelectionCombo(parent);
		
		return null;
	}
	
	/*
	 * currently disabled tree selection composite
	 */
	private Control createTreeSelectionCombo(Composite parent){
		Composite taxonomicTreeSelection = new Composite(parent, SWT.NULL);
		taxonomicTreeSelection.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		GridLayout layout = new GridLayout();
		taxonomicTreeSelection.setLayout(layout);
		
		Label label = new Label(taxonomicTreeSelection, SWT.NULL);
		// TODO not working is not really true but leave it there to remind everyone that this is under construction
		label.setText("Select Taxonomic Tree (not completely working at the moment)");
		taxonomicTreeSelectionCombo = new Combo(taxonomicTreeSelection, SWT.BORDER | SWT.READ_ONLY);
		taxonomicTreeSelectionCombo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true));
		
		for(TaxonomicTree tree : taxonomicTrees){
			taxonomicTreeSelectionCombo.add(tree.getName().getText(), taxonomicTrees.indexOf(tree));
		}
		
		// TODO remember last selection
		
		taxonomicTreeSelectionCombo.addSelectionListener(new SelectionAdapter(){

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectedTaxonomicTree = taxonomicTrees.get(taxonomicTreeSelectionCombo.getSelectionIndex());
			}
			
		});
		
		return taxonomicTreeSelection;
	}
}