// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageStringBase;
import eu.etaxonomy.cdm.model.common.OriginalSource;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.location.NamedArea;

/**
 * @author p.ciardelli
 * @created 02.04.2009
 * @version 1.0
 */
public class DescriptionHelper {
	private static final Logger logger = Logger
			.getLogger(DescriptionHelper.class);

	/**
	 * Returns whatever the element's title cache equivalent is,
	 * depending on its class.
	 * 
	 * @param element
	 * @param language
	 * @return 
	 */
	public static String getCache(DescriptionElementBase element, 
			Language language) {
		String cache = null;
		if (element instanceof TextData) {
			cache = ((TextData) element).getText(language);
		}
		if (element instanceof CommonTaxonName) {
			cache = ((CommonTaxonName) element).getName();
		}
		if (element instanceof Distribution) {
			Distribution distribution = (Distribution) element;
			
			NamedArea area = distribution.getArea();
			PresenceAbsenceTermBase<?> status = distribution.getStatus();
			
			if (status != null) {
				cache =  area.getLabel() + ", " + status.getLabel();
			}
		}
		return cache == null ? "" : cache;
	}

	/**
	 * Returns whatever the element's title cache equivalent is,
	 * depending on its class, using the default language.
	 * 
	 * @param element
	 * @return 
	 */
	public static String getCache(DescriptionElementBase element) {
		return getCache(element, Language.DEFAULT());
	}
	
	/**
	 * Set whatever the element's title cache equivalent is,
	 * depending on its class.
	 * 
	 * @param element
	 * @param value
	 * @param language
	 */
	public static void setCache(DescriptionElementBase element,
			String value, Language language) {
		if (element instanceof TextData) {
			((TextData) element).putText(value, language);
			return;
		}
		if (element instanceof CommonTaxonName) {
			((CommonTaxonName) element).setName(value);
			return;
		}
		if(element instanceof Distribution){
			logger.warn("trying to set cache on distribution, don't know what to do at the moment.");
			return;
		}
		logger.warn("No matching subclass found for DescriptionElementBase object, 'cache' not set.");
	}
	
	/**
	 * Set whatever the element's title cache equivalent is,
	 * depending on its class, using the default language.
	 * 
	 * @param element
	 * @param value
	 */	
	public static void setCache(DescriptionElementBase element,
			String value) {
		setCache(element, value, Language.DEFAULT());
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.referencingobjects.IReferencingObjectsService#getObjectDescription(java.lang.Object)
	 */
	public static String getObjectDescription(Object element) {
		if (element instanceof IdentifiableEntity) {
			return ((IdentifiableEntity) element).getTitleCache();
		}
		if (element instanceof OriginalSource) {
			OriginalSource originalSource = (OriginalSource) element;
//			ReferenceBase citation = HibernateProxyHelper.deproxy(originalSource.getCitation(), ReferenceBase.class);
			return CdmUtils.concat(";", new String[]{originalSource.getIdNamespace(), originalSource.getIdInSource()});
		}
		if (element instanceof LanguageStringBase) {
			return ((LanguageStringBase) element).getText();
		}
		if (element instanceof DescriptionElementBase) {
			return getCache((DescriptionElementBase) element);		
		}
		// TODO write return texts for NameRelationship, HomotypicalGroup, SpecimenTypeDesignation, etc.
		return element.toString();
	}
}
