// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;

/**
 * @author p.ciardelli
 * @created 11.09.2009
 * @version 1.0
 */
public class NomenclaturalCodeHelper {
	private static final Logger logger = Logger
			.getLogger(NomenclaturalCodeHelper.class);
		
	/**
	 * @return
	 */
	public static List<NomenclaturalCode> getAllCodes() {
		return Arrays.asList(NomenclaturalCode.values());
	}


	/**
	 * @return
	 */
	public static List<NomenclaturalCode> getImplementedCodes() {
		List<NomenclaturalCode> implementedCodes = new ArrayList<NomenclaturalCode>();
		for (NomenclaturalCode code : getAllCodes()) {
			if (getIsImplemented(code)) {
				implementedCodes.add(code);
			}
		}
		return implementedCodes;
	}
	
	/**
	 * @param code
	 * @return
	 */
	private static boolean getIsImplemented(NomenclaturalCode code) {
		if (code.equals(NomenclaturalCode.ICBN) || code.equals(NomenclaturalCode.ICZN)) {
			return true;
		} else {
			return false;
		}
	}


	/**
	 * @param code
	 * @return 
	 */
	public static String getDescription(NomenclaturalCode code) {
		if (code.equals(NomenclaturalCode.ICNB)) {
			return "International Code of Nomenclature of Bacteria (ICNB)";
		}
		if (code.equals(NomenclaturalCode.ICBN)) {
			return "International Code of Botanical Nomenclature (ICBN)";
		}
		if (code.equals(NomenclaturalCode.ICNCP)) {
			return "International Code of Cultivated Plants (ICNCP)";
		}
		if (code.equals(NomenclaturalCode.ICZN)) {
			return "International Code of Zoological Nomenclature (ICZN)";
		} 		
		if (code.equals(NomenclaturalCode.ICVCN)) {
			return "International Code for Virus Classification and Nomenclature (ICVCN)";
		} 
		return code.name();
	}
	
	/**
	 * @return
	 */
	public static NomenclaturalCode getDefaultCode() {
		return NomenclaturalCode.ICBN;
	}
}
