// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 23.06.2009
 * @version 1.0
 */
public class CreateTaxonomicTreeOperation extends AbstractPostOperation {

	private static final Logger logger = Logger
			.getLogger(CreateTaxonomicTreeOperation.class);
	private String treeLabel;
	private UUID referenceUuid;
	private String microReference;
	
	/**
	 * @param label
	 * @param undoContext 
	 */
	public CreateTaxonomicTreeOperation(String label, IUndoContext undoContext, String treeLabel, UUID referenceUuid, String microReference) {
		super(label, undoContext);
		
		this.treeLabel = treeLabel;
		this.referenceUuid = referenceUuid;
		this.microReference = microReference;
	
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		if(treeLabel.trim().length() == 0){
			return Status.CANCEL_STATUS;
		}
		
		return postExecute(createTaxonomicTreeIsolated());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}
	
	private TaxonomicTree createTaxonomicTreeIsolated(){
		// get a new conversation
		ConversationHolder conversation = CdmStore.NewTransactionalConversation();
		
		TaxonomicTree tree = TaxonomicTree.NewInstance(treeLabel);
		ReferenceBase reference;
		if(referenceUuid == null){
			reference = Generic.NewInstance();
			reference.setTitleCache(treeLabel);
		}else{
			reference = CdmStore.getReferenceService().getReferenceByUuid(referenceUuid);
		}
		
		tree.setReference(reference);
		tree.setMicroReference(microReference);
		
		CdmStore.getTaxonService().saveTaxonomicTree(tree);
		
		conversation.commit(true);
		
		return tree;
	}
}
