/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.parser;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.strategy.parser.INonViralNameParser;
import eu.etaxonomy.cdm.strategy.parser.NonViralNameParserImpl;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author p.ciardelli
 * @created 22.12.2008
 * @version 1.0
 */
public class CdmParserUtil {
	private static final Logger logger = Logger
			.getLogger(CdmParserUtil.class);
	
	private static INonViralNameParser nonViralNameParser;
	
	private static INonViralNameParser getNonViralNameParser() {
		if (nonViralNameParser == null) {
			nonViralNameParser = NonViralNameParserImpl.NewInstance();
		}
		return nonViralNameParser;
	}
	
	public static void clearNonViralNameParser() {
		nonViralNameParser = null;
	}
	
	/**
	 * Tries to create a taxonomic name from a given string
	 * 
	 * @param unparsedNameString
	 * @param rank
	 * @return
	 */
	public static TaxonNameBase parseReferencedName(String unparsedNameString, Rank rank){
		NomenclaturalCode nomCode = PreferencesUtil.getPreferredNomenclaturalCode();
		
		TaxonNameBase name = getNonViralNameParser().parseReferencedName(unparsedNameString,
				nomCode, rank);

		if (name.hasProblem()) {
			name.setFullTitleCache(unparsedNameString);
		}
		
		return name;
	}
	
	public static TaxonNameBase parseReferencedName(String unparsedNameString){
		return parseReferencedName(unparsedNameString, null);
	}
	
	public static TaxonNameBase createEmptyName(){
		return parseReferencedName("");
	}
	
}
