// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference.wizards;

import org.apache.log4j.Logger;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.Representation;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author n.hoffmann
 * @created 12.06.2009
 * @version 1.0
 */
public class VocabularyTermWizard<T extends DefinedTermBase> extends Wizard {
	private static final Logger logger = Logger.getLogger(VocabularyTermWizard.class);
	
	public T term = null;
	
	private VocabularyEnum vocabulary;
	
	private boolean editMode = false;

	/**
	 * 
	 */
	public VocabularyTermWizard(VocabularyEnum vocabulary){		
		this.vocabulary = vocabulary;
	}
	
	/**
	 * 
	 * @param selectedFeature
	 */
	public VocabularyTermWizard(VocabularyEnum vocabulary, T selectedTerm){
		this(vocabulary);
		this.term = selectedTerm;
		editMode = true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		addPage(new VocabularyTermWizardPage());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		ConversationHolder conversation = CdmStore.NewTransactionalConversation();
		
		if(editMode){
			VocabularyStore.updateVocabulary(getVocabulary());
		}else{
			VocabularyStore.saveTerm(getTerm(), getVocabulary());
		}
		
		conversation.commit(true);
		
		return true;
	}

	public VocabularyEnum getVocabulary(){
		return vocabulary;
	}

	/**
	 * 
	 * @param term
	 */
	public void setTerm(T term) {
		this.term = term;
	}

	/**
	 * @param text
	 * @param text2
	 * @param text3
	 */
	public void setTerm(String termString, String label, String labelAbbrev) {
		Class clazz = vocabulary.getClazz();
		
		try {
			term = (T) clazz.newInstance();
			
			term.addRepresentation(new Representation(termString, label, labelAbbrev, Language.DEFAULT()));
			
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	/**
	 * @return the term
	 */
	public T getTerm() {
		return term;
	}

	/**
	 * @return the editMode
	 */
	public boolean isEditMode() {
		return editMode;
	}

}
