// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store;

import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.ExtensionType;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.OrderedTermBase;
import eu.etaxonomy.cdm.model.common.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;

/**
 * All vocabularies may be accessed through this store.
 * Note: This is for internal use. When using vocabularies in the UI, try to use the preferred terms from
 * PreferenceUtil
 * 
 * @author n.hoffmann
 * @created 25.06.2009
 * @version 1.0
 */
public class VocabularyStore {

	/**
	 * The singleton instance
	 */
	private static VocabularyStore instance;
	
	/**
	 * Retrieve the singleton instance 
	 * 
	 * @return
	 */
	private static synchronized VocabularyStore getDefault(){
		if(instance == null){
			instance = new VocabularyStore();
		}
		return instance;
	}
	
	/**
	 * 
	 * @return
	 */
	public static SortedSet<SpecimenTypeDesignationStatus> getSpecimenTypeDesignationStatus() {
		SortedSet specimenTypeDesignationStatus = getDefault().getTermVocabulary(VocabularyEnum.SpecimenTypeDesignationStatus).getTermsOrderedByLabels(getDefaultLanguage());
		return specimenTypeDesignationStatus;
	}
	
	/**
	 * 
	 * @return
	 */
	public static SortedSet<TaxonRelationshipType> getTaxonRelationshipTypes(){
		SortedSet taxonRelationshipTypes = getDefault().getOrderedTermVocabulary(VocabularyEnum.TaxonRelationshipType).getOrderedTerms();
		return taxonRelationshipTypes;
	}
	
	/**
	 * 
	 * @return
	 * 
	 * @deprecated handle via preferences
	 */
	public static SortedSet<TaxonRelationshipType> getConceptRelationshipTypes() {
		SortedSet<TaxonRelationshipType> conceptRelationshipTypes = getTaxonRelationshipTypes();
		// remove these two relations as they are considered standard taxon relations
		conceptRelationshipTypes.remove(TaxonRelationshipType.MISAPPLIED_NAME_FOR());
		conceptRelationshipTypes.remove(TaxonRelationshipType.TAXONOMICALLY_INCLUDED_IN());
	
		return conceptRelationshipTypes;
	}
	
	/**
	 * 
	 * @return
	 */
	public static Set<Language> getLanguages(){

		Set languages = getDefault().getTermVocabulary(VocabularyEnum.Language).getTerms();
		
//		SortedSet languages = getDefault().getTermVocabulary(VocabularyEnum.Language).getTermsOrderedByLabels(getDefaultLanguage());
		return languages;
	}
	
	public static SortedSet<? extends DefinedTermBase> getTerms(VocabularyEnum vocabularyType){
		
		TermVocabulary<DefinedTermBase> terms = getDefault().getTermVocabulary(vocabularyType);
		
		if(terms != null) {
			return terms.getTermsOrderedByLabels(getDefaultLanguage());
		}
		return null;
	}
	
	/**
	 * @param selected
	 */
	public static void delete(DefinedTermBase selected) {
		getDefault().getTermService().delete(selected);
	}
	/**
	 * 
	 * @return
	 */
	public static SortedSet<Feature> getFeatures() {
		SortedSet features = getDefault().getTermVocabulary(VocabularyEnum.Feature).getTermsOrderedByLabels(getDefaultLanguage());
		features.remove(Feature.IMAGE());
		return features;

	}
	
	/**
	 * 
	 * @return
	 */
	public static SortedSet<NameRelationshipType> getNameRelationshipTypes() {
		SortedSet nameRelationshipTypes = getDefault().getTermVocabulary(VocabularyEnum.NameRelationshipType).getTermsOrderedByLabels(getDefaultLanguage());
		return nameRelationshipTypes;
	}
	
	/**
	 * 
	 * @return
	 */
	public static Set<NomenclaturalStatusType> getNomenclaturalStatusTypes(){
		// TODO sort types
		Set nomenclaturalStatusTypes = getDefault().getTermVocabulary(VocabularyEnum.NomenclaturalStatusType).getTermsOrderedByLabels(getDefaultLanguage());
		return nomenclaturalStatusTypes;
	}
	

	/**
	 * @return 
	 * 
	 */
	public static Set<MarkerType> getMarkerTypes() {
		Set markerTypes = getDefault().getTermVocabulary(VocabularyEnum.MarkerType).getTermsOrderedByLabels(getDefaultLanguage());
		return markerTypes;
	}
	

	/**
	 * @return
	 */
	public static Set<ExtensionType> getExtensionTypes() {
		Set extensionTypes = getDefault().getTermVocabulary(VocabularyEnum.ExtensionType).getTermsOrderedByLabels(getDefaultLanguage());
		return extensionTypes;
	}
	
	/**
	 * Retrieve all ranks from data store.
	 * 
	 * @return
	 */
	public static SortedSet<Rank> getRanks(){
		SortedSet ranks = getDefault().getOrderedTermVocabulary(VocabularyEnum.Rank).getOrderedTerms();
		return ranks;
	}
	
	
	/**
	 * Retrieve all presence and absence terms from data store.
	 *  
	 * @return
	 */
	public static SortedSet<PresenceAbsenceTermBase<PresenceAbsenceTermBase<?>>> getPresenceAbsenceTerms() {
		SortedSet presenceAbsenceTerms = getDefault().getTermVocabulary(VocabularyEnum.PresenceTerm).getTermsOrderedByLabels(getDefaultLanguage());
			presenceAbsenceTerms.addAll(getDefault().getTermVocabulary(VocabularyEnum.AbsenceTerm).getTermsOrderedByLabels(getDefaultLanguage()));

		return presenceAbsenceTerms;
	}

	
	/*************** save methods ******************************/
	/**
	 * Adds a term to a vocabulary
	 * 
	 * @param feature
	 */
	public static void saveTerm(DefinedTermBase term, VocabularyEnum vocabularyType) {
		TermVocabulary<DefinedTermBase> vocabulary = getDefault().getTermVocabulary(vocabularyType);
		
		vocabulary.addTerm(term);
		
		getDefault().getTermService().saveTermVocabulary(vocabulary);
	}
	
	/**
	 * Save a vacabulary to data store
	 * 
	 * @param feature
	 */
	public static void updateVocabulary(VocabularyEnum vocabularyType) {
		TermVocabulary<DefinedTermBase> vocabulary = getDefault().getTermVocabulary(vocabularyType);

		getDefault().getTermService().saveTermVocabulary(vocabulary);
	}
	
	
	
	/*************** internal methods **************************/
	
	private static Language getDefaultLanguage(){
		return CdmStore.getDefaultLanguage();
	}
	
	private OrderedTermVocabulary<? extends OrderedTermBase> getOrderedTermVocabulary(VocabularyEnum vocabularyType){
 		TermVocabulary<DefinedTermBase> vocabulary = getTermVocabulary(vocabularyType);		
 		return HibernateProxyHelper.deproxy(vocabulary, OrderedTermVocabulary.class);
	}
	
	private TermVocabulary<DefinedTermBase> getTermVocabulary(VocabularyEnum vocabularyType){
		return getTermService().getVocabulary(vocabularyType);
	}
	
	private ITermService getTermService(){
		return getApplicationController().getTermService();
	}
	
	private CdmApplicationController getApplicationController(){
		return CdmStore.getApplicationControllerWillBeObsolete();
	}

}
