// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store.operations;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.operations.DeleteTaxonNodeOperation;

/**
 * @author n.hoffmann
 * @created 08.04.2009
 * @version 1.0
 */
public class DeleteTaxonNodeOperationTest extends AbstractTaxeditorOperationTest{
	private static final Logger logger = Logger
			.getLogger(DeleteTaxonNodeOperationTest.class);
	
	private static Taxon child;
	
	private static TaxonNode taxonNode;
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		// TODO get taxon from integration test data
		taxon = Taxon.NewInstance(null, null);
		
		child = Taxon.NewInstance(null, null);
		
		taxon.addTaxonomicChild(child, null, null);
		
		operation = new DeleteTaxonNodeOperation("Delete Taxon", undoContext, taxonNode);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.DeleteTaxonNodeOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testExecuteIProgressMonitorIAdaptable() throws ExecutionException {

		operation.execute(monitor, info);
	
		Assert.assertTrue(taxon.getTaxonomicChildren().size() == 0);
		Assert.assertTrue(taxon.getTaxonomicChildrenCount() == 0);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.DeleteTaxonNodeOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testUndoIProgressMonitorIAdaptable() throws ExecutionException {

		operation.execute(monitor, info);
	
		Assert.assertTrue(taxon.getTaxonomicChildren().size() > 0);
		Assert.assertTrue(taxon.getTaxonomicChildrenCount() > 0);
	}
	
	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.DeleteTaxonNodeOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testRedoIProgressMonitorIAdaptable() throws ExecutionException {

		operation.execute(monitor, info);
	
		Assert.assertTrue(taxon.getTaxonomicChildren().size() == 0);
		Assert.assertTrue(taxon.getTaxonomicChildrenCount() == 0);
	}
}
